@@ -26,21 +26,11 @@
 
 #include <gst/gst_private.h>
 
-#ifndef G_OS_WIN32
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#else
-#define WIN32_LEAN_AND_MEAN
-
-#define fsync(fd) _commit(fd)
-#include <io.h>
-
-#include <windows.h>
-extern HMODULE _priv_gst_dll_handle;
-#endif
 
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
@@ -147,7 +137,7 @@ static gboolean plugin_loader_sync_with_child (GstPluginLoader * l);
 static GstPluginLoader *
 plugin_loader_new (GstRegistry * registry)
 {
-  GstPluginLoader *l = g_slice_new0 (GstPluginLoader);
+  GstPluginLoader *l = g_new0 (GstPluginLoader, 1);
 
   if (registry)
     l->registry = gst_object_ref (registry);
@@ -211,12 +201,12 @@ plugin_loader_free (GstPluginLoader * loader)
   while (cur) {
     PendingPluginEntry *entry = (PendingPluginEntry *) (cur->data);
     g_free (entry->filename);
-    g_slice_free (PendingPluginEntry, entry);
+    g_free (entry);
 
     cur = g_list_delete_link (cur, cur);
   }
 
-  g_slice_free (GstPluginLoader, loader);
+  g_free (loader);
 
   return got_plugin_details;
 }
@@ -235,7 +225,7 @@ plugin_loader_load (GstPluginLoader * loader, const gchar * filename,
   GST_LOG_OBJECT (loader->registry,
       "Sending file %s to child. tag %u", filename, loader->next_tag);
 
-  entry = g_slice_new (PendingPluginEntry);
+  entry = g_new (PendingPluginEntry, 1);
   entry->tag = loader->next_tag++;
   entry->filename = g_strdup (filename);
   entry->file_size = file_size;
@@ -274,7 +264,12 @@ restart:
   while ((cur = l->pending_plugins)) {
     PendingPluginEntry *entry = (PendingPluginEntry *) (cur->data);
 
-    if (!plugin_loader_load_and_sync (l, entry)) {
+    /* Maybe the cur(current head of the pending plugins list) has already been
+     * processed and deleted from pending plugins in handle_rx_packet when
+     * received this plugin details, so here add cur == l->pending_plugins
+     * to ensure that the cur is valid */
+    if (!plugin_loader_load_and_sync (l, entry)
+        && cur == l->pending_plugins) {
       /* Create dummy plugin entry to block re-scanning this file */
       GST_ERROR ("Plugin file %s failed to load. Blacklisting",
           entry->filename);
@@ -283,7 +278,7 @@ restart:
       /* Now remove this crashy plugin from the head of the list */
       l->pending_plugins = g_list_delete_link (cur, cur);
       g_free (entry->filename);
-      g_slice_free (PendingPluginEntry, entry);
+      g_free (entry);
       if (l->pending_plugins == NULL)
         l->pending_plugins_tail = NULL;
       if (!gst_plugin_loader_spawn (l))
@@ -413,6 +408,7 @@ gst_plugin_loader_try_helper (GstPluginLoader * loader, gchar * location)
 {
   char *argv[6] = { NULL, };
   int c = 0;
+  GError *error = NULL;
 
 #if defined (__APPLE__) && defined (USR_BIN_ARCH_SWITCH)
   if (gst_plugin_loader_use_usr_bin_arch ()) {
@@ -432,11 +428,16 @@ gst_plugin_loader_try_helper (GstPluginLoader * loader, gchar * location)
     GST_LOG ("Trying to spawn gst-plugin-scanner helper at %s", location);
   }
 
-  if (!g_spawn_async_with_pipes (NULL, argv, NULL,
-          G_SPAWN_DO_NOT_REAP_CHILD /* | G_SPAWN_STDERR_TO_DEV_NULL */ ,
-          NULL, NULL, &loader->child_pid, &loader->fd_w.fd, &loader->fd_r.fd,
-          NULL, NULL))
+  g_spawn_async_with_pipes (NULL, argv, NULL,
+      G_SPAWN_DO_NOT_REAP_CHILD /* | G_SPAWN_STDERR_TO_DEV_NULL */ ,
+      NULL, NULL, &loader->child_pid, &loader->fd_w.fd, &loader->fd_r.fd,
+      NULL, &error);
+
+  if (error) {
+    GST_ERROR ("Spawning gst-plugin-scanner helper failed: %s", error->message);
+    g_clear_error (&error);
     return FALSE;
+  }
 
   gst_poll_add_fd (loader->fdset, &loader->fd_w);
   gst_poll_add_fd (loader->fdset, &loader->fd_r);
@@ -481,23 +482,44 @@ gst_plugin_loader_spawn (GstPluginLoader * loader)
     /* use the installed version */
     GST_LOG ("Trying installed plugin scanner");
 
-#ifdef G_OS_WIN32
-#define EXESUFFIX ".exe"
-#else
-#define EXESUFFIX
-#endif
+#define MAX_PATH_DEPTH 64
 
     relocated_libgstreamer = priv_gst_get_relocated_libgstreamer ();
     if (relocated_libgstreamer) {
+      int plugin_subdir_depth = priv_gst_count_directories (GST_PLUGIN_SUBDIR);
+
       GST_DEBUG ("found libgstreamer-" GST_API_VERSION " library "
           "at %s", relocated_libgstreamer);
-      helper_bin = g_build_filename (relocated_libgstreamer,
-          "..", GST_PLUGIN_SCANNER_SUBDIR, "gstreamer-" GST_API_VERSION,
-          "gst-plugin-scanner" EXESUFFIX, NULL);
+
+      if (plugin_subdir_depth < MAX_PATH_DEPTH) {
+        const char *filenamev[MAX_PATH_DEPTH + 5];
+        int i = 0, j;
+
+        filenamev[i++] = relocated_libgstreamer;
+        for (j = 0; j < plugin_subdir_depth; j++)
+          filenamev[i++] = "..";
+        filenamev[i++] = GST_PLUGIN_SCANNER_SUBDIR;
+        filenamev[i++] = "gstreamer-" GST_API_VERSION;
+        filenamev[i++] = "gst-plugin-scanner";
+        filenamev[i++] = NULL;
+        g_assert (i <= MAX_PATH_DEPTH + 5);
+
+        GST_DEBUG ("constructing path to system plugin scanner using "
+            "plugin dir: \'%s\', plugin scanner dir: \'%s\'",
+            GST_PLUGIN_SUBDIR, GST_PLUGIN_SCANNER_SUBDIR);
+
+        helper_bin = g_build_filenamev ((char **) filenamev);
+      } else {
+        GST_WARNING ("GST_PLUGIN_SUBDIR: \'%s\' has too many path segments",
+            GST_PLUGIN_SUBDIR);
+        helper_bin = g_strdup (GST_PLUGIN_SCANNER_INSTALLED);
+      }
     } else {
       helper_bin = g_strdup (GST_PLUGIN_SCANNER_INSTALLED);
     }
 
+#undef MAX_PATH_DEPTH
+
     GST_DEBUG ("using system plugin scanner at %s", helper_bin);
 
     res = gst_plugin_loader_try_helper (loader, helper_bin);
@@ -524,22 +546,19 @@ plugin_loader_cleanup_child (GstPluginLoader * l)
   close (l->fd_w.fd);
   close (l->fd_r.fd);
 
-#ifndef G_OS_WIN32
   GST_LOG ("waiting for child process to exit");
   waitpid (l->child_pid, NULL, 0);
-#else
-  g_warning ("FIXME: Implement child process shutdown for Win32");
-#endif
   g_spawn_close_pid (l->child_pid);
 
   l->child_running = FALSE;
 }
 
 gboolean
-_gst_plugin_loader_client_run (void)
+_gst_plugin_loader_client_run (const gchar * pipe_name)
 {
   gboolean res = TRUE;
   GstPluginLoader *l;
+  int dup_fd;
 
   l = plugin_loader_new (NULL);
   if (l == NULL)
@@ -548,37 +567,27 @@ _gst_plugin_loader_client_run (void)
   /* On entry, the inward pipe is STDIN, and outward is STDOUT.
    * Dup those somewhere better so that plugins printing things
    * won't interfere with anything */
-#ifndef G_OS_WIN32
-  {
-    int dup_fd;
-
-    dup_fd = dup (0);           /* STDIN */
-    if (dup_fd == -1) {
-      GST_ERROR ("Failed to start. Could not dup STDIN, errno %d", errno);
-      res = FALSE;
-      goto beach;
-    }
-    l->fd_r.fd = dup_fd;
-    close (0);
-
-    dup_fd = dup (1);           /* STDOUT */
-    if (dup_fd == -1) {
-      GST_ERROR ("Failed to start. Could not dup STDOUT, errno %d", errno);
-      res = FALSE;
-      goto beach;
-    }
-    l->fd_w.fd = dup_fd;
-    close (1);
-
-    /* Dup stderr down to stdout so things that plugins print are visible,
-     * but don't care if it fails */
-    dup2 (2, 1);
+  dup_fd = dup (0);             /* STDIN */
+  if (dup_fd == -1) {
+    GST_ERROR ("Failed to start. Could not dup STDIN, errno %d", errno);
+    res = FALSE;
+    goto beach;
   }
-#else
-  /* FIXME: Use DuplicateHandle and friends on win32 */
-  l->fd_w.fd = 1;               /* STDOUT */
-  l->fd_r.fd = 0;               /* STDIN */
-#endif
+  l->fd_r.fd = dup_fd;
+  close (0);
+
+  dup_fd = dup (1);             /* STDOUT */
+  if (dup_fd == -1) {
+    GST_ERROR ("Failed to start. Could not dup STDOUT, errno %d", errno);
+    res = FALSE;
+    goto beach;
+  }
+  l->fd_w.fd = dup_fd;
+  close (1);
+
+  /* Dup stderr down to stdout so things that plugins print are visible,
+   * but don't care if it fails */
+  dup2 (2, 1);
 
   gst_poll_add_fd (l->fdset, &l->fd_w);
   gst_poll_add_fd (l->fdset, &l->fd_r);
@@ -591,10 +600,7 @@ _gst_plugin_loader_client_run (void)
   /* Loop, listening for incoming packets on the fd and writing responses */
   while (!l->rx_done && exchange_packets (l));
 
-#ifndef G_OS_WIN32
 beach:
-#endif
-
   plugin_loader_free (l);
 
   return res;
@@ -702,6 +708,9 @@ write_one (GstPluginLoader * l)
         continue;
       /* Failed to write -> child died */
       goto fail_and_cleanup;
+    } else if (G_UNLIKELY (res == 0)) {
+      /* FD closed -> child died */
+      goto fail_and_cleanup;
     }
     to_write -= res;
     out += res;
@@ -871,7 +880,7 @@ handle_rx_packet (GstPluginLoader * l,
         } else {
           cur = g_list_delete_link (cur, cur);
           g_free (e->filename);
-          g_slice_free (PendingPluginEntry, e);
+          g_free (e);
         }
       }
 
@@ -905,7 +914,7 @@ handle_rx_packet (GstPluginLoader * l,
 
       if (entry != NULL) {
         g_free (entry->filename);
-        g_slice_free (PendingPluginEntry, entry);
+        g_free (entry);
       }
 
       /* Remove the plugin entry we just loaded */
@@ -966,6 +975,9 @@ read_one (GstPluginLoader * l)
         continue;
       GST_LOG ("Failed reading packet header");
       return FALSE;
+    } else if (G_UNLIKELY (res == 0)) {
+      GST_LOG ("Failed reading packet header: Unexpected EOF");
+      return FALSE;
     }
     to_read -= res;
     in += res;
@@ -1003,6 +1015,9 @@ read_one (GstPluginLoader * l)
           continue;
         GST_ERROR ("Packet payload read failed");
         return FALSE;
+      } else if (G_UNLIKELY (res == 0)) {
+        GST_ERROR ("Packet payload read failed: Unexpected EOF");
+        return FALSE;
       }
       to_read -= res;
       in += res;
