@@ -21,6 +21,7 @@
 
 /**
  * SECTION:gsttagsetter
+ * @title: GstTagsetter
  * @short_description: Element interface that allows setting and retrieval
  *                     of media metadata
  *
@@ -29,7 +30,7 @@
  * Elements that support changing a stream's metadata will implement this
  * interface. Examples of such elements are 'vorbisenc', 'theoraenc' and
  * 'id3v2mux'.
- * 
+ *
  * If you just want to retrieve metadata in your application then all you
  * need to do is watch for tag messages on your pipeline's bus. This
  * interface is only for setting metadata, not for extracting it. To set tags
@@ -38,30 +39,30 @@
  * setting the #GstTagMergeMode that is used for tag events that arrive at the
  * tagsetter element (default mode is to keep existing tags).
  * The application should do that before the element goes to %GST_STATE_PAUSED.
- * 
+ *
  * Elements implementing the #GstTagSetter interface often have to merge
  * any tags received from upstream and the tags set by the application via
  * the interface. This can be done like this:
  *
- * |[
+ * |[<!-- language="C" -->
  * GstTagMergeMode merge_mode;
  * const GstTagList *application_tags;
  * const GstTagList *event_tags;
  * GstTagSetter *tagsetter;
  * GstTagList *result;
- *  
+ *
  * tagsetter = GST_TAG_SETTER (element);
- *  
+ *
  * merge_mode = gst_tag_setter_get_tag_merge_mode (tagsetter);
  * application_tags = gst_tag_setter_get_tag_list (tagsetter);
  * event_tags = (const GstTagList *) element->event_tags;
- *  
+ *
  * GST_LOG_OBJECT (tagsetter, "merging tags, merge mode = %d", merge_mode);
  * GST_LOG_OBJECT (tagsetter, "event tags: %" GST_PTR_FORMAT, event_tags);
  * GST_LOG_OBJECT (tagsetter, "set   tags: %" GST_PTR_FORMAT, application_tags);
- *  
+ *
  * result = gst_tag_list_merge (application_tags, event_tags, merge_mode);
- *  
+ *
  * GST_LOG_OBJECT (tagsetter, "final tags: %" GST_PTR_FORMAT, result);
  * ]|
  */
@@ -107,7 +108,7 @@ gst_tag_data_free (gpointer p)
 
   g_mutex_clear (&data->lock);
 
-  g_slice_free (GstTagData, data);
+  g_free (data);
 }
 
 static GstTagData *
@@ -124,7 +125,7 @@ gst_tag_setter_get_data (GstTagSetter * setter)
 
     data = g_object_get_qdata (G_OBJECT (setter), gst_tag_key);
     if (!data) {
-      data = g_slice_new (GstTagData);
+      data = g_new (GstTagData, 1);
       g_mutex_init (&data->lock);
       data->list = NULL;
       data->mode = GST_TAG_MERGE_KEEP;
