@@ -106,8 +106,8 @@
  * since then can be queried with gst_adapter_distance_from_discont().
  *
  * A last thing to note is that while #GstAdapter is pretty optimized,
- * merging buffers still might be an operation that requires a malloc() and
- * memcpy() operation, and these operations are not the fastest. Because of
+ * merging buffers still might be an operation that requires a `malloc()` and
+ * `memcpy()` operation, and these operations are not the fastest. Because of
  * this, some functions like gst_adapter_available_fast() are provided to help
  * speed up such cases should you want to. To avoid repeated memory allocations,
  * gst_adapter_copy() can be used to copy data into a (statically allocated)
@@ -127,6 +127,7 @@
 #include <gst/gst_private.h>
 #include "gstadapter.h"
 #include <string.h>
+#include <gst/base/gstqueuearray.h>
 
 /* default size for the assembled data buffer */
 #define DEFAULT_SIZE 4096
@@ -141,8 +142,7 @@ struct _GstAdapter
   GObject object;
 
   /*< private > */
-  GSList *buflist;
-  GSList *buflist_end;
+  GstVecDeque *bufqueue;
   gsize size;
   gsize skip;
   guint count;
@@ -160,7 +160,8 @@ struct _GstAdapter
   guint64 offset_distance;
 
   gsize scan_offset;
-  GSList *scan_entry;
+  /* G_MAXUINT when unset */
+  guint scan_entry_idx;
 
   GstClockTime pts_at_discont;
   GstClockTime dts_at_discont;
@@ -208,6 +209,7 @@ gst_adapter_init (GstAdapter * adapter)
   adapter->dts_at_discont = GST_CLOCK_TIME_NONE;
   adapter->offset_at_discont = GST_BUFFER_OFFSET_NONE;
   adapter->distance_from_discont = 0;
+  adapter->bufqueue = gst_vec_deque_new (10);
 }
 
 static void
@@ -227,6 +229,8 @@ gst_adapter_finalize (GObject * object)
 
   g_free (adapter->assembled_data);
 
+  gst_vec_deque_free (adapter->bufqueue);
+
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -252,15 +256,15 @@ gst_adapter_new (void)
 void
 gst_adapter_clear (GstAdapter * adapter)
 {
+  GstMiniObject *obj;
   g_return_if_fail (GST_IS_ADAPTER (adapter));
 
   if (adapter->info.memory)
     gst_adapter_unmap (adapter);
 
-  g_slist_foreach (adapter->buflist, (GFunc) gst_mini_object_unref, NULL);
-  g_slist_free (adapter->buflist);
-  adapter->buflist = NULL;
-  adapter->buflist_end = NULL;
+  while ((obj = gst_vec_deque_pop_head (adapter->bufqueue)))
+    gst_mini_object_unref (obj);
+
   adapter->count = 0;
   adapter->size = 0;
   adapter->skip = 0;
@@ -276,7 +280,7 @@ gst_adapter_clear (GstAdapter * adapter)
   adapter->offset_at_discont = GST_BUFFER_OFFSET_NONE;
   adapter->distance_from_discont = 0;
   adapter->scan_offset = 0;
-  adapter->scan_entry = NULL;
+  adapter->scan_entry_idx = G_MAXUINT;
 }
 
 static inline void
@@ -318,24 +322,23 @@ static void
 copy_into_unchecked (GstAdapter * adapter, guint8 * dest, gsize skip,
     gsize size)
 {
-  GSList *g;
   GstBuffer *buf;
   gsize bsize, csize;
+  guint idx = 0;
 
   /* first step, do skipping */
   /* we might well be copying where we were scanning */
-  if (adapter->scan_entry && (adapter->scan_offset <= skip)) {
-    g = adapter->scan_entry;
+  if (adapter->scan_entry_idx != G_MAXUINT && (adapter->scan_offset <= skip)) {
+    idx = adapter->scan_entry_idx;
     skip -= adapter->scan_offset;
   } else {
-    g = adapter->buflist;
+    idx = 0;
   }
-  buf = g->data;
+  buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
   bsize = gst_buffer_get_size (buf);
   while (G_UNLIKELY (skip >= bsize)) {
     skip -= bsize;
-    g = g_slist_next (g);
-    buf = g->data;
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
   }
   /* copy partial buffer */
@@ -350,8 +353,7 @@ copy_into_unchecked (GstAdapter * adapter, guint8 * dest, gsize skip,
 
   /* second step, copy remainder */
   while (size > 0) {
-    g = g_slist_next (g);
-    buf = g->data;
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
     if (G_LIKELY (bsize > 0)) {
       csize = MIN (bsize, size);
@@ -384,17 +386,16 @@ gst_adapter_push (GstAdapter * adapter, GstBuffer * buf)
   adapter->size += size;
 
   /* Note: merging buffers at this point is premature. */
-  if (G_UNLIKELY (adapter->buflist == NULL)) {
+  if (gst_vec_deque_is_empty (adapter->bufqueue)) {
     GST_LOG_OBJECT (adapter, "pushing %p first %" G_GSIZE_FORMAT " bytes",
         buf, size);
-    adapter->buflist = adapter->buflist_end = g_slist_append (NULL, buf);
+    gst_vec_deque_push_tail (adapter->bufqueue, buf);
     update_timestamps_and_offset (adapter, buf);
   } else {
     /* Otherwise append to the end, and advance our end pointer */
     GST_LOG_OBJECT (adapter, "pushing %p %" G_GSIZE_FORMAT " bytes at end, "
         "size now %" G_GSIZE_FORMAT, buf, size, adapter->size);
-    adapter->buflist_end = g_slist_append (adapter->buflist_end, buf);
-    adapter->buflist_end = g_slist_next (adapter->buflist_end);
+    gst_vec_deque_push_tail (adapter->bufqueue, buf);
   }
   ++adapter->count;
 }
@@ -506,7 +507,7 @@ gst_adapter_map (GstAdapter * adapter, gsize size)
 #if 0
   do {
 #endif
-    cur = adapter->buflist->data;
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
 
     csize = gst_buffer_get_size (cur);
@@ -567,7 +568,7 @@ gst_adapter_unmap (GstAdapter * adapter)
   g_return_if_fail (GST_IS_ADAPTER (adapter));
 
   if (adapter->info.memory) {
-    GstBuffer *cur = adapter->buflist->data;
+    GstBuffer *cur = gst_vec_deque_peek_head (adapter->bufqueue);
     GST_LOG_OBJECT (adapter, "unmap memory buffer %p", cur);
     gst_buffer_unmap (cur, &adapter->info);
     adapter->info.memory = NULL;
@@ -580,7 +581,7 @@ gst_adapter_unmap (GstAdapter * adapter)
  * @dest: (out caller-allocates) (array length=size) (element-type guint8):
  *     the memory to copy into
  * @offset: the bytes offset in the adapter to start from
- * @size: the number of bytes to copy
+ * @size: (in): the number of bytes to copy
  *
  * Copies @size bytes of data starting at @offset out of the buffers
  * contained in #GstAdapter into an array @dest provided by the caller.
@@ -629,7 +630,6 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
 {
   GstBuffer *cur;
   gsize size;
-  GSList *g;
 
   GST_LOG_OBJECT (adapter, "flushing %" G_GSIZE_FORMAT " bytes", flush);
 
@@ -648,8 +648,7 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
   adapter->offset_distance -= adapter->skip;
   adapter->distance_from_discont -= adapter->skip;
 
-  g = adapter->buflist;
-  cur = g->data;
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
   size = gst_buffer_get_size (cur);
   while (flush >= size) {
     /* can skip whole buffer */
@@ -660,21 +659,20 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
     adapter->distance_from_discont += size;
     flush -= size;
 
-    gst_buffer_unref (cur);
-    g = g_slist_delete_link (g, g);
     --adapter->count;
 
-    if (G_UNLIKELY (g == NULL)) {
+    cur = NULL;
+    gst_buffer_unref (gst_vec_deque_pop_head (adapter->bufqueue));
+
+    if (gst_vec_deque_is_empty (adapter->bufqueue)) {
       GST_LOG_OBJECT (adapter, "adapter empty now");
-      adapter->buflist_end = NULL;
       break;
     }
     /* there is a new head buffer, update the timestamps */
-    cur = g->data;
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     update_timestamps_and_offset (adapter, cur);
     size = gst_buffer_get_size (cur);
   }
-  adapter->buflist = g;
   /* account for the remaining bytes */
   adapter->skip = flush;
   adapter->pts_distance += flush;
@@ -683,7 +681,7 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
   adapter->distance_from_discont += flush;
   /* invalidate scan position */
   adapter->scan_offset = 0;
-  adapter->scan_entry = NULL;
+  adapter->scan_entry_idx = G_MAXUINT;
 }
 
 /**
@@ -813,9 +811,9 @@ gst_adapter_get_buffer_fast (GstAdapter * adapter, gsize nbytes)
 {
   GstBuffer *buffer = NULL;
   GstBuffer *cur;
-  GSList *item;
   gsize skip;
   gsize left = nbytes;
+  guint idx, len;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), NULL);
   g_return_val_if_fail (nbytes > 0, NULL);
@@ -830,7 +828,7 @@ gst_adapter_get_buffer_fast (GstAdapter * adapter, gsize nbytes)
     return NULL;
 
   skip = adapter->skip;
-  cur = adapter->buflist->data;
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
 
   if (skip == 0 && gst_buffer_get_size (cur) == nbytes) {
     GST_LOG_OBJECT (adapter, "providing buffer of %" G_GSIZE_FORMAT " bytes"
@@ -839,10 +837,12 @@ gst_adapter_get_buffer_fast (GstAdapter * adapter, gsize nbytes)
     goto done;
   }
 
-  for (item = adapter->buflist; item && left > 0; item = item->next) {
+  len = gst_vec_deque_get_length (adapter->bufqueue);
+
+  for (idx = 0; idx < len && left > 0; idx++) {
     gsize size, cur_size;
 
-    cur = item->data;
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx);
     cur_size = gst_buffer_get_size (cur);
     size = MIN (cur_size - skip, left);
 
@@ -919,7 +919,8 @@ foreach_metadata (GstBuffer * inbuf, GstMeta ** meta, gpointer user_data)
   const GstMetaInfo *info = (*meta)->info;
   gboolean do_copy = FALSE;
 
-  if (gst_meta_api_type_has_tag (info->api, _gst_meta_tag_memory)) {
+  if (gst_meta_api_type_has_tag (info->api, _gst_meta_tag_memory)
+      || gst_meta_api_type_has_tag (info->api, _gst_meta_tag_memory_reference)) {
     /* never call the transform_meta with memory specific metadata */
     GST_DEBUG ("not copying memory specific metadata %s",
         g_type_name (info->api));
@@ -929,7 +930,7 @@ foreach_metadata (GstBuffer * inbuf, GstMeta ** meta, gpointer user_data)
     GST_DEBUG ("copying metadata %s", g_type_name (info->api));
   }
 
-  if (do_copy) {
+  if (do_copy && info->transform_func) {
     GstMetaTransformCopy copy_data = { FALSE, 0, -1 };
     GST_DEBUG ("copy metadata %s", g_type_name (info->api));
     /* simply copy then */
@@ -979,7 +980,7 @@ gst_adapter_get_buffer (GstAdapter * adapter, gsize nbytes)
   if (G_UNLIKELY (nbytes > adapter->size))
     return NULL;
 
-  cur = adapter->buflist->data;
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
   skip = adapter->skip;
   hsize = gst_buffer_get_size (cur);
 
@@ -1014,18 +1015,20 @@ gst_adapter_get_buffer (GstAdapter * adapter, gsize nbytes)
   buffer = gst_buffer_new_wrapped (data, nbytes);
 
   {
-    GSList *g;
+    guint idx, len;
     GstBuffer *cur;
     gsize read_offset = 0;
 
-    g = adapter->buflist;
-    while (g && read_offset < nbytes + adapter->skip) {
-      cur = g->data;
+    idx = 0;
+    len = gst_vec_deque_get_length (adapter->bufqueue);
+
+    while (idx < len && read_offset < nbytes + adapter->skip) {
+      cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx);
 
       gst_buffer_foreach_meta (cur, foreach_metadata, buffer);
       read_offset += gst_buffer_get_size (cur);
 
-      g = g_slist_next (g);
+      idx++;
     }
   }
 
@@ -1108,7 +1111,7 @@ gst_adapter_take_list (GstAdapter * adapter, gsize nbytes)
   GST_LOG_OBJECT (adapter, "taking %" G_GSIZE_FORMAT " bytes", nbytes);
 
   while (nbytes > 0) {
-    cur = adapter->buflist->data;
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
@@ -1146,18 +1149,18 @@ gst_adapter_get_list (GstAdapter * adapter, gsize nbytes)
   GQueue queue = G_QUEUE_INIT;
   GstBuffer *cur, *buffer;
   gsize hsize, skip, cur_size;
-  GSList *g = NULL;
+  guint idx;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), NULL);
   g_return_val_if_fail (nbytes <= adapter->size, NULL);
 
   GST_LOG_OBJECT (adapter, "getting %" G_GSIZE_FORMAT " bytes", nbytes);
 
-  g = adapter->buflist;
+  idx = 0;
   skip = adapter->skip;
 
   while (nbytes > 0) {
-    cur = g->data;
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
 
@@ -1175,7 +1178,6 @@ gst_adapter_get_list (GstAdapter * adapter, gsize nbytes)
 
     nbytes -= hsize;
     skip = 0;
-    g = g_slist_next (g);
   }
 
   return queue.head;
@@ -1224,7 +1226,7 @@ gst_adapter_take_buffer_list (GstAdapter * adapter, gsize nbytes)
   buffer_list = gst_buffer_list_new_sized (n_bufs);
 
   while (nbytes > 0) {
-    cur = adapter->buflist->data;
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
@@ -1260,7 +1262,7 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
   GstBuffer *cur, *buffer;
   gsize hsize, skip, cur_size;
   guint n_bufs;
-  GSList *g = NULL;
+  guint idx;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), NULL);
 
@@ -1277,11 +1279,11 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
 
   buffer_list = gst_buffer_list_new_sized (n_bufs);
 
-  g = adapter->buflist;
+  idx = 0;
   skip = adapter->skip;
 
   while (nbytes > 0) {
-    cur = g->data;
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
 
@@ -1299,7 +1301,6 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
 
     nbytes -= hsize;
     skip = 0;
-    g = g_slist_next (g);
   }
 
   return buffer_list;
@@ -1313,6 +1314,10 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
  * value that can be supplied to gst_adapter_map() without that function
  * returning %NULL.
  *
+ * Calling gst_adapter_map() with the amount of bytes returned by this function
+ * may require expensive operations (like copying the data into a temporary
+ * buffer) in some cases.
+ *
  * Returns: number of bytes available in @adapter
  */
 gsize
@@ -1327,8 +1332,11 @@ gst_adapter_available (GstAdapter * adapter)
  * gst_adapter_available_fast:
  * @adapter: a #GstAdapter
  *
- * Gets the maximum number of bytes that are immediately available without
- * requiring any expensive operations (like copying the data into a
+ * Gets the maximum number of bytes that can be retrieved in a single map
+ * operation without merging buffers.
+ *
+ * Calling gst_adapter_map() with the amount of bytes returned by this function
+ * will never require any expensive operations (like copying the data into a
  * temporary buffer).
  *
  * Returns: number of bytes that are available in @adapter without expensive
@@ -1339,7 +1347,7 @@ gst_adapter_available_fast (GstAdapter * adapter)
 {
   GstBuffer *cur;
   gsize size;
-  GSList *g;
+  guint idx;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), 0);
 
@@ -1352,13 +1360,12 @@ gst_adapter_available_fast (GstAdapter * adapter)
     return adapter->assembled_len;
 
   /* take the first non-zero buffer */
-  g = adapter->buflist;
+  idx = 0;
   while (TRUE) {
-    cur = g->data;
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     size = gst_buffer_get_size (cur);
     if (size != 0)
       break;
-    g = g_slist_next (g);
   }
 
   /* we can quickly get the (remaining) data of the first buffer */
@@ -1366,7 +1373,7 @@ gst_adapter_available_fast (GstAdapter * adapter)
 }
 
 /**
- * gst_adapter_get_distance_from_discont:
+ * gst_adapter_distance_from_discont:
  * @adapter: a #GstAdapter
  *
  * Get the distance in bytes since the last buffer with the
@@ -1454,7 +1461,7 @@ gst_adapter_dts_at_discont (GstAdapter * adapter)
  *
  * The offset is reset to GST_BUFFER_OFFSET_NONE and the distance is set to 0
  * when the adapter is first created or when it is cleared. This also means that
- * before the first byte with an offset is removed from the adapter, the offset
+ * before the first byte with an offset is added to the adapter, the offset
  * and distance returned are GST_BUFFER_OFFSET_NONE and 0 respectively.
  *
  * Since: 1.10
@@ -1483,7 +1490,7 @@ gst_adapter_prev_offset (GstAdapter * adapter, guint64 * distance)
  *
  * The pts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a pts is removed from the adapter, the pts
+ * the first byte with a pts is added to the adapter, the pts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Returns: The previously seen pts.
@@ -1510,7 +1517,7 @@ gst_adapter_prev_pts (GstAdapter * adapter, guint64 * distance)
  *
  * The dts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a dts is removed from the adapter, the dts
+ * the first byte with a dts is added to the adapter, the dts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Returns: The previously seen dts.
@@ -1538,7 +1545,7 @@ gst_adapter_prev_dts (GstAdapter * adapter, guint64 * distance)
  *
  * The pts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a pts is removed from the adapter, the pts
+ * the first byte with a pts is added to the adapter, the pts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Since: 1.2
@@ -1549,17 +1556,18 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
     guint64 * distance)
 {
   GstBuffer *cur;
-  GSList *g;
   gsize read_offset = 0;
   gsize pts_offset = 0;
   GstClockTime pts = adapter->pts;
+  guint idx, len;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
 
-  g = adapter->buflist;
+  idx = 0;
+  len = gst_vec_deque_get_length (adapter->bufqueue);
 
-  while (g && read_offset < offset + adapter->skip) {
-    cur = g->data;
+  while (idx < len && read_offset < offset + adapter->skip) {
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (cur))) {
       pts = GST_BUFFER_PTS (cur);
@@ -1567,7 +1575,6 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
     }
 
     read_offset += gst_buffer_get_size (cur);
-    g = g_slist_next (g);
   }
 
   if (distance)
@@ -1588,7 +1595,7 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
  *
  * The dts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a dts is removed from the adapter, the dts
+ * the first byte with a dts is added to the adapter, the dts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Since: 1.2
@@ -1599,17 +1606,18 @@ gst_adapter_prev_dts_at_offset (GstAdapter * adapter, gsize offset,
     guint64 * distance)
 {
   GstBuffer *cur;
-  GSList *g;
   gsize read_offset = 0;
   gsize dts_offset = 0;
   GstClockTime dts = adapter->dts;
+  guint idx, len;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
 
-  g = adapter->buflist;
+  idx = 0;
+  len = gst_vec_deque_get_length (adapter->bufqueue);
 
-  while (g && read_offset < offset + adapter->skip) {
-    cur = g->data;
+  while (idx < len && read_offset < offset + adapter->skip) {
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (cur))) {
       dts = GST_BUFFER_DTS (cur);
@@ -1617,7 +1625,6 @@ gst_adapter_prev_dts_at_offset (GstAdapter * adapter, gsize offset,
     }
 
     read_offset += gst_buffer_get_size (cur);
-    g = g_slist_next (g);
   }
 
   if (distance)
@@ -1653,12 +1660,12 @@ gssize
 gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
     guint32 pattern, gsize offset, gsize size, guint32 * value)
 {
-  GSList *g;
   gsize skip, bsize, i;
   guint32 state;
   GstMapInfo info;
   guint8 *bdata;
   GstBuffer *buf;
+  guint idx;
 
   g_return_val_if_fail (size > 0, -1);
   g_return_val_if_fail (offset + size <= adapter->size, -1);
@@ -1672,22 +1679,21 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
 
   /* first step, do skipping and position on the first buffer */
   /* optimistically assume scanning continues sequentially */
-  if (adapter->scan_entry && (adapter->scan_offset <= skip)) {
-    g = adapter->scan_entry;
+  if (adapter->scan_entry_idx != G_MAXUINT && (adapter->scan_offset <= skip)) {
+    idx = adapter->scan_entry_idx;
     skip -= adapter->scan_offset;
   } else {
-    g = adapter->buflist;
+    idx = 0;
     adapter->scan_offset = 0;
-    adapter->scan_entry = NULL;
+    adapter->scan_entry_idx = G_MAXUINT;
   }
-  buf = g->data;
+  buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
   bsize = gst_buffer_get_size (buf);
   while (G_UNLIKELY (skip >= bsize)) {
     skip -= bsize;
-    g = g_slist_next (g);
     adapter->scan_offset += bsize;
-    adapter->scan_entry = g;
-    buf = g->data;
+    adapter->scan_entry_idx = idx;
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
   }
   /* get the data now */
@@ -1723,11 +1729,10 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
 
     /* nothing found yet, go to next buffer */
     skip += bsize;
-    g = g_slist_next (g);
     adapter->scan_offset += info.size;
-    adapter->scan_entry = g;
+    adapter->scan_entry_idx = idx;
     gst_buffer_unmap (buf, &info);
-    buf = g->data;
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (!gst_buffer_map (buf, &info, GST_MAP_READ))
       return -1;
