@@ -21,27 +21,28 @@
  */
 /**
  * SECTION:element-capsfilter
+ * @title: capsfilter
+ * @short_description: enforce limitations on the data format
  *
- * The element does not modify data as such, but can enforce limitations on the
- * data format.
+ * The element does not modify data as such, but can enforce limitations on the data format.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 videotestsrc ! capsfilter caps=video/x-raw,format=GRAY8 ! videoconvert ! autovideosink
  * ]| Limits acceptable video from videotestsrc to be grayscale. Equivalent to
  * |[
  * gst-launch-1.0 videotestsrc ! video/x-raw,format=GRAY8 ! videoconvert ! autovideosink
  * ]| which is a short notation for the capsfilter element.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "../../gst/gst-i18n-lib.h"
+#include <glib/gi18n-lib.h>
 #include "gstcapsfilter.h"
+#include "gstcoreelementselements.h"
 
 enum
 {
@@ -93,6 +94,8 @@ gst_caps_filter_caps_change_mode_get_type (void)
 G_DEFINE_TYPE_WITH_CODE (GstCapsFilter, gst_capsfilter, GST_TYPE_BASE_TRANSFORM,
     _do_init);
 
+GST_ELEMENT_REGISTER_DEFINE (capsfilter, "capsfilter", GST_RANK_NONE,
+    GST_TYPE_CAPS_FILTER);
 
 static void gst_capsfilter_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -145,10 +148,8 @@ gst_capsfilter_class_init (GstCapsFilterClass * klass)
       "Generic",
       "Pass data without modification, limiting formats",
       "David Schleef <ds@schleef.org>");
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&srctemplate));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
+  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
 
   trans_class = GST_BASE_TRANSFORM_CLASS (klass);
   trans_class->transform_caps =
@@ -159,6 +160,8 @@ gst_capsfilter_class_init (GstCapsFilterClass * klass)
       GST_DEBUG_FUNCPTR (gst_capsfilter_prepare_buf);
   trans_class->sink_event = GST_DEBUG_FUNCPTR (gst_capsfilter_sink_event);
   trans_class->stop = GST_DEBUG_FUNCPTR (gst_capsfilter_stop);
+
+  gst_type_mark_as_plugin_api (GST_TYPE_CAPS_FILTER_CAPS_CHANGE_MODE, 0);
 }
 
 static void
@@ -169,6 +172,7 @@ gst_capsfilter_init (GstCapsFilter * filter)
   gst_base_transform_set_prefer_passthrough (trans, FALSE);
   filter->filter_caps = gst_caps_new_any ();
   filter->filter_caps_used = FALSE;
+  filter->got_sink_caps = FALSE;
   filter->caps_change_mode = DEFAULT_CAPS_CHANGE_MODE;
 }
 
@@ -386,7 +390,7 @@ gst_capsfilter_push_pending_events (GstCapsFilter * filter, GList * events)
   g_list_free (events);
 }
 
-/* Ouput buffer preparation ... if the buffer has no caps, and our allowed
+/* Output buffer preparation ... if the buffer has no caps, and our allowed
  * output caps is fixed, then send the caps downstream, making sure caps are
  * sent before segment event.
  *
@@ -406,7 +410,7 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
   *buf = input;
 
   if (GST_PAD_MODE (trans->srcpad) == GST_PAD_MODE_PUSH
-      && !gst_pad_has_current_caps (trans->sinkpad)) {
+      && !filter->got_sink_caps) {
 
     /* No caps. See if the output pad only supports fixed caps */
     GstCaps *out_caps;
@@ -507,7 +511,7 @@ gst_capsfilter_sink_event (GstBaseTransform * trans, GstEvent * event)
     }
     g_list_free (filter->pending_events);
     filter->pending_events = NULL;
-  } else if (!gst_pad_has_current_caps (trans->sinkpad)) {
+  } else if (!filter->got_sink_caps) {
     GST_LOG_OBJECT (trans, "Got %s event before caps, queueing",
         GST_EVENT_TYPE_NAME (event));
 
@@ -523,32 +527,35 @@ done:
       GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans,
       gst_event_ref (event));
 
-  if (GST_EVENT_TYPE (event) == GST_EVENT_CAPS
-      && filter->caps_change_mode == GST_CAPS_FILTER_CAPS_CHANGE_MODE_DELAYED) {
-    GList *l;
-    GstCaps *caps;
+  if (GST_EVENT_TYPE (event) == GST_EVENT_CAPS) {
+    filter->got_sink_caps = TRUE;
+    if (filter->caps_change_mode == GST_CAPS_FILTER_CAPS_CHANGE_MODE_DELAYED) {
+      GList *l;
+      GstCaps *caps;
 
-    gst_event_parse_caps (event, &caps);
+      gst_event_parse_caps (event, &caps);
 
-    /* Remove all previous caps up to one that works.
-     * Note that this might keep some leftover caps if there
-     * are multiple compatible caps */
-    GST_OBJECT_LOCK (filter);
-    for (l = g_list_last (filter->previous_caps); l; l = l->prev) {
-      if (gst_caps_can_intersect (caps, l->data)) {
-        while (l->next) {
-          gst_caps_unref (l->next->data);
-          l = g_list_delete_link (l, l->next);
+      /* Remove all previous caps up to one that works.
+       * Note that this might keep some leftover caps if there
+       * are multiple compatible caps */
+      GST_OBJECT_LOCK (filter);
+      for (l = g_list_last (filter->previous_caps); l; l = l->prev) {
+        if (gst_caps_can_intersect (caps, l->data)) {
+          while (l->next) {
+            gst_caps_unref (l->next->data);
+            l = g_list_delete_link (l, l->next);
+          }
+          break;
         }
-        break;
       }
+      if (!l && gst_caps_can_intersect (caps, filter->filter_caps)) {
+        g_list_free_full (filter->previous_caps,
+            (GDestroyNotify) gst_caps_unref);
+        filter->previous_caps = NULL;
+        filter->filter_caps_used = TRUE;
+      }
+      GST_OBJECT_UNLOCK (filter);
     }
-    if (!l && gst_caps_can_intersect (caps, filter->filter_caps)) {
-      g_list_free_full (filter->previous_caps, (GDestroyNotify) gst_caps_unref);
-      filter->previous_caps = NULL;
-      filter->filter_caps_used = TRUE;
-    }
-    GST_OBJECT_UNLOCK (filter);
   }
   gst_event_unref (event);
 
@@ -568,5 +575,7 @@ gst_capsfilter_stop (GstBaseTransform * trans)
   filter->previous_caps = NULL;
   GST_OBJECT_UNLOCK (filter);
 
+  filter->got_sink_caps = FALSE;
+
   return TRUE;
 }
