@@ -21,8 +21,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-
-#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/base/gstpushsrc.h>
 #include <gst/check/gstcheck.h>
@@ -889,6 +890,175 @@ GST_START_TEST (test_position)
 
 GST_END_TEST;
 
+/* test position reporting still works after a FLUSH_STOP(reset_time=FALSE) */
+GST_START_TEST (test_position_non_resetting_flush)
+{
+  GstElement *pipeline, *sink;
+  GstPad *sinkpad;
+  GstStateChangeReturn ret;
+  gboolean qret;
+  gint64 qcur;
+  GstBuffer *buffer;
+  GstFlowReturn fret;
+  ChainData *data;
+  GstEvent *event;
+  gboolean eret;
+
+  /* create sink */
+  pipeline = gst_pipeline_new ("pipeline");
+  fail_if (pipeline == NULL);
+
+  sink = gst_element_factory_make ("fakesink", "sink");
+  fail_if (sink == NULL);
+  g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
+  g_object_set (G_OBJECT (sink), "num-buffers", 2, NULL);
+
+  gst_bin_add (GST_BIN (pipeline), sink);
+
+  sinkpad = gst_element_get_static_pad (sink, "sink");
+  fail_if (sinkpad == NULL);
+
+  /* make pipeline and element ready to accept data */
+  ret = gst_element_set_state (pipeline, GST_STATE_PAUSED);
+  fail_unless (ret == GST_STATE_CHANGE_ASYNC);
+
+  gst_pad_send_event (sinkpad, gst_event_new_stream_start ("test"));
+
+  /* send segment, this should work */
+  {
+    GstSegment segment;
+
+    GST_DEBUG ("sending segment");
+    gst_segment_init (&segment, GST_FORMAT_TIME);
+    segment.start = 1 * GST_SECOND;
+    segment.stop = 3 * GST_SECOND;
+    segment.time = 1 * GST_SECOND;
+    event = gst_event_new_segment (&segment);
+
+    eret = gst_pad_send_event (sinkpad, event);
+    fail_if (eret == FALSE);
+  }
+
+  /* do position query, this should succeed with the time value from the segment. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 1 * GST_SECOND);
+
+  /* send buffer that we will flush out */
+  buffer = gst_buffer_new ();
+  GST_BUFFER_PTS (buffer) = 2 * GST_SECOND;
+  GST_BUFFER_DURATION (buffer) = 1 * GST_SECOND;
+
+  GST_DEBUG ("sending buffer");
+
+  /* this buffer causes the sink to preroll */
+  data = chain_async (sinkpad, buffer);
+  fail_if (data == NULL);
+
+  /* wait for preroll */
+  ret = gst_element_get_state (pipeline, NULL, NULL, -1);
+
+  /* do position query, this should succeed with the time value from the
+   * segment. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 1 * GST_SECOND);
+
+  /* start flushing */
+  {
+    GST_DEBUG ("sending flush_start");
+    event = gst_event_new_flush_start ();
+
+    eret = gst_pad_send_event (sinkpad, event);
+    fail_if (eret == FALSE);
+  }
+
+  /* preroll buffer is flushed out */
+  fret = chain_async_return (data);
+  fail_unless (fret == GST_FLOW_FLUSHING);
+
+  /* do position query, this should succeed with the time value from the
+   * segment before the flush. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 1 * GST_SECOND);
+
+  /* send a FLUSH_STOP with reset_time=FALSE */
+  {
+    GST_DEBUG ("sending flush_stop");
+    event = gst_event_new_flush_stop (FALSE);
+
+    eret = gst_pad_send_event (sinkpad, event);
+    fail_if (eret == FALSE);
+  }
+
+  /* do position query again, this should again succeed with the time
+   * value from the segment before the flush. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 1 * GST_SECOND);
+
+  /* send a different segment, this should work */
+  {
+    GstSegment segment;
+
+    GST_DEBUG ("sending segment");
+    gst_segment_init (&segment, GST_FORMAT_TIME);
+    segment.start = 2 * GST_SECOND;
+    segment.stop = 4 * GST_SECOND;
+    segment.time = 10 * GST_SECOND;
+    event = gst_event_new_segment (&segment);
+
+    eret = gst_pad_send_event (sinkpad, event);
+    fail_if (eret == FALSE);
+  }
+
+  /* do position query again, the position should reflect the time
+   * value from the new segment. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 10 * GST_SECOND);
+
+  /* send buffer that we will flush out */
+  buffer = gst_buffer_new ();
+  GST_BUFFER_PTS (buffer) = 2 * GST_SECOND;
+  GST_BUFFER_DURATION (buffer) = 1 * GST_SECOND;
+
+  GST_DEBUG ("sending buffer");
+
+  /* this buffer causes the sink to preroll */
+  data = chain_async (sinkpad, buffer);
+  fail_if (data == NULL);
+
+  /* wait for preroll */
+  ret = gst_element_get_state (pipeline, NULL, NULL, -1);
+
+  /* do position query, this should succeed with the time value from the
+   * segment. */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == TRUE);
+  fail_unless (qcur == 10 * GST_SECOND);
+
+  ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  fail_unless (ret == GST_STATE_CHANGE_SUCCESS);
+
+  /* preroll buffer is rendered */
+  fret = chain_async_return (data);
+  fail_unless (fret == GST_FLOW_OK);
+
+  ret = gst_element_set_state (pipeline, GST_STATE_NULL);
+  fail_unless (ret == GST_STATE_CHANGE_SUCCESS);
+
+  /* fails because we are in the wrong state */
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &qcur);
+  fail_unless (qret == FALSE);
+
+  gst_object_unref (sinkpad);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 /* like fakesrc, but also pushes an OOB event after each buffer */
 typedef GstPushSrc OOBSource;
 typedef GstPushSrcClass OOBSourceClass;
@@ -915,8 +1085,7 @@ oob_source_class_init (OOBSourceClass * klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstPushSrcClass *pushsrc_class = GST_PUSH_SRC_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
 
   pushsrc_class->create = GST_DEBUG_FUNCPTR (oob_source_create);
 }
@@ -953,12 +1122,13 @@ test_notify_race_setup_pipeline (NotifyRacePipeline * p)
   gst_bin_add (GST_BIN (p->pipe), p->sink);
   gst_element_link_many (p->src, p->queue, p->sink, NULL);
 
-  GST_DEBUG ("Setting pipeline to PLAYING");
-  fail_unless_equals_int (gst_element_set_state (p->pipe, GST_STATE_PLAYING),
+  GST_DEBUG ("Setting pipeline to PAUSED..");
+  fail_unless_equals_int (gst_element_set_state (p->pipe, GST_STATE_PAUSED),
       GST_STATE_CHANGE_ASYNC);
-  GST_DEBUG ("Getting state");
+  GST_DEBUG ("Waiting for pipeline to preroll..");
   fail_unless_equals_int (gst_element_get_state (p->pipe, NULL, NULL, -1),
       GST_STATE_CHANGE_SUCCESS);
+  GST_DEBUG ("Ready to party!");
 }
 
 static void
@@ -977,10 +1147,15 @@ GST_START_TEST (test_notify_race)
   int i;
 
   for (i = 0; i < G_N_ELEMENTS (pipelines); ++i) {
-    GST_DEBUG ("Starting up pipeline %d", i);
+    GST_DEBUG ("Setting up pipeline %d", i);
     test_notify_race_setup_pipeline (&pipelines[i]);
   }
 
+  for (i = 0; i < G_N_ELEMENTS (pipelines); ++i) {
+    GST_DEBUG ("Starting pipeline %d", i);
+    gst_element_set_state (pipelines[i].pipe, GST_STATE_PLAYING);
+  }
+
   g_usleep (2 * G_USEC_PER_SEC);
 
   for (i = 0; i < G_N_ELEMENTS (pipelines); ++i) {
@@ -1002,7 +1177,7 @@ last_message_cb (GObject * obj, GParamSpec * pspec, gpointer user_data)
   fail_unless (last_msg != NULL);
 
   if (!strstr (last_msg, "chain"))
-    return;
+    goto skip;
 
   GST_LOG_OBJECT (obj, "%s", last_msg);
 
@@ -1021,6 +1196,8 @@ last_message_cb (GObject * obj, GParamSpec * pspec, gpointer user_data)
 
   *p_counter = count + 1;
 
+skip:
+
   g_free (last_msg);
 }
 
@@ -1070,6 +1247,8 @@ GST_START_TEST (test_last_message_notify)
 
   fail_unless_equals_int (counter1, NUM_BUFFERS);
   fail_unless_equals_int (counter2, NUM_BUFFERS);
+
+  gst_object_unref (pipe);
 }
 
 GST_END_TEST;
@@ -1094,7 +1273,7 @@ deep_notify_last_message_cb (GstObject * obj, GstObject * prop_obj,
   fail_unless (last_msg != NULL);
 
   if (!strstr (last_msg, "chain"))
-    return;
+    goto skip;
 
   GST_LOG_OBJECT (prop_obj, "%s", last_msg);
 
@@ -1113,6 +1292,8 @@ deep_notify_last_message_cb (GstObject * obj, GstObject * prop_obj,
 
   *p_counter = count + 1;
 
+skip:
+
   g_free (last_msg);
 }
 
@@ -1178,6 +1359,7 @@ fakesink_suite (void)
   tcase_add_test (tc_chain, test_eos);
   tcase_add_test (tc_chain, test_eos2);
   tcase_add_test (tc_chain, test_position);
+  tcase_add_test (tc_chain, test_position_non_resetting_flush);
   tcase_add_test (tc_chain, test_notify_race);
   tcase_add_test (tc_chain, test_last_message_notify);
   tcase_skip_broken_test (tc_chain, test_last_message_deep_notify);
